@@ -2,14 +2,12 @@
 Our app setting
 """
 
+# Standard Library
+from re import RegexFlag
+
 # Django
 from django.apps import apps
-from django.utils.text import slugify
-
-# Alliance Auth AFAT
-from afat import __title__
-
-AFAT_BASE_URL = slugify(value=__title__, allow_unicode=True)
+from django.conf import settings
 
 
 def fittings_installed() -> bool:
@@ -23,6 +21,17 @@ def fittings_installed() -> bool:
     return apps.is_installed(app_name="fittings")
 
 
+def securegroups_installed() -> bool:
+    """
+    Check if the Alliance Auth Secure Groups module is installed
+
+    :return:
+    :rtype:
+    """
+
+    return apps.is_installed(app_name="securegroups")
+
+
 def use_fittings_module_for_doctrines() -> bool:
     """
     Check if the Fittings module is used for doctrines
@@ -41,3 +50,14 @@ def use_fittings_module_for_doctrines() -> bool:
         and Setting.get_setting(Setting.Field.USE_DOCTRINES_FROM_FITTINGS_MODULE)
         is True
     )
+
+
+def debug_enabled() -> RegexFlag:
+    """
+    Check if DEBUG is enabled
+
+    :return:
+    :rtype:
+    """
+
+    return settings.DEBUG
