@@ -7,8 +7,10 @@ from allianceauth import hooks
 from allianceauth.services.hooks import MenuItemHook, UrlHook
 
 # Alliance Auth AFAT
-from afat import __title__, urls
-from afat.app_settings import AFAT_BASE_URL
+from afat import __title_translated__, urls
+from afat.app_settings import securegroups_installed
+from afat.constants import APP_BASE_URL
+from afat.models import FatsInTimeFilter
 
 
 class AaAfatMenuItem(MenuItemHook):  # pylint: disable=too-few-public-methods
@@ -20,7 +22,7 @@ class AaAfatMenuItem(MenuItemHook):  # pylint: disable=too-few-public-methods
         # Setup menu entry for sidebar
         MenuItemHook.__init__(
             self,
-            text=__title__,
+            text=__title_translated__,
             classes="fa-solid fa-space-shuttle",
             url_name="afat:dashboard",
             navactive=["afat:"],
@@ -63,4 +65,18 @@ def register_url():
     :rtype:
     """
 
-    return UrlHook(urls=urls, namespace="afat", base_url=rf"^{AFAT_BASE_URL}/")
+    return UrlHook(urls=urls, namespace="afat", base_url=rf"^{APP_BASE_URL}/")
+
+
+if securegroups_installed():
+
+    @hooks.register("secure_group_filters")
+    def filters() -> list:
+        """
+        Secure group filter
+
+        :return: Secure group filters
+        :rtype: list
+        """
+
+        return [FatsInTimeFilter]
