@@ -1,32 +1,21 @@
 """
-Versioned static URLs to break browser caches when changing the app version
+Versioned static URLs to break browser caches when changing the app version.
 """
 
 # Django
 from django.template.defaulttags import register
-from django.templatetags.static import static
 from django.utils.translation import gettext as _
 
+# Alliance Auth
+from allianceauth.services.hooks import get_extension_logger
+
+# Alliance Auth (External Libs)
+from app_utils.logging import LoggerAddTag
+
 # Alliance Auth AFAT
-from afat import __version__
+from afat import __title__
 
-
-@register.simple_tag
-def afat_static(path: str) -> str:
-    """
-    Versioned static URL
-    example: {% afat_static 'afat/css/allianceauth-afat.min.css' %}
-
-    :param path:
-    :type path:
-    :return:
-    :rtype:
-    """
-
-    static_url = static(path=path)
-    versioned_url = static_url + "?v=" + __version__
-
-    return versioned_url
+logger = LoggerAddTag(my_logger=get_extension_logger(__name__), prefix=__title__)
 
 
 @register.filter
