@@ -4,13 +4,12 @@ Test our template tags
 
 # Django
 from django.template import Context, Template
-from django.test import TestCase
 
 # Alliance Auth AFAT
-from afat import __version__
+from afat.tests import BaseTestCase
 
 
-class TestAfatFilters(TestCase):
+class TestAfatFilters(BaseTestCase):
     """
     Test template filters
     """
@@ -32,35 +31,7 @@ class TestAfatFilters(TestCase):
         self.assertInHTML(needle="May", haystack=rendered_template)
 
 
-class TestAfatStatic(TestCase):
-    """
-    Test versioned static template tag
-    """
-
-    def test_afat_static(self):
-        """
-        Test afat_static
-
-        :return:
-        """
-
-        context = Context(dict_={"version": __version__})
-        template_to_render = Template(
-            template_string=(
-                "{% load afat %}"
-                "{% afat_static 'afat/css/allianceauth-afat.min.css' %}"
-            )
-        )
-
-        rendered_template = template_to_render.render(context=context)
-
-        self.assertInHTML(
-            needle=f'/static/afat/css/allianceauth-afat.min.css?v={context["version"]}',
-            haystack=rendered_template,
-        )
-
-
-class SumValuesFilterTest(TestCase):
+class TestSumValuesFilter(BaseTestCase):
     """
     Test the sum_values filter
     """
