@@ -1,13 +1,12 @@
 # Standard Library
-import datetime as dt
 from http import HTTPStatus
 
 # Third Party
 from pytz import utc
 
 # Django
-from django.test import TestCase
 from django.urls import reverse
+from django.utils.datetime_safe import datetime
 
 # Alliance Auth
 from allianceauth.eveonline.models import EveCharacter
@@ -17,6 +16,7 @@ from app_utils.testing import add_character_to_user, create_user_from_evecharact
 
 # Alliance Auth AFAT
 from afat.models import Fat, FatLink
+from afat.tests import BaseTestCase
 from afat.tests.fixtures.load_allianceauth import load_allianceauth
 from afat.tests.fixtures.utils import RequestStub
 from afat.views.statistics import _calculate_year_stats
@@ -25,16 +25,35 @@ MODULE_PATH = "afat.views.statistics"
 
 
 def response_content_to_str(response) -> str:
+    """
+    Convert response content to string
+
+    :param response:
+    :type response:
+    :return:
+    :rtype:
+    """
+
     return response.content.decode(response.charset)
 
 
-class TestStatistics(TestCase):
+class TestStatistics(BaseTestCase):
+    """
+    Test the statistics views
+    """
+
     @classmethod
     def setUpClass(cls):
+        """
+        Setup the test class
+
+        :return:
+        :rtype:
+        """
+
         super().setUpClass()
         load_allianceauth()
 
-        # given
         cls.character_1001 = EveCharacter.objects.get(character_id=1001)
         cls.character_1002 = EveCharacter.objects.get(character_id=1002)
         cls.character_1003 = EveCharacter.objects.get(character_id=1003)
@@ -76,21 +95,21 @@ class TestStatistics(TestCase):
             hash="1231",
             creator=cls.user_with_basic_access,
             character=cls.character_1001,
-            created=dt.datetime(year=2020, month=4, day=1, tzinfo=utc),
+            created=datetime(year=2020, month=4, day=1, tzinfo=utc),
         )
         afat_link_april_2 = FatLink.objects.create(
             fleet="April Fleet 2",
             hash="1232",
             creator=cls.user_with_basic_access,
             character=cls.character_1001,
-            created=dt.datetime(year=2020, month=4, day=15, tzinfo=utc),
+            created=datetime(year=2020, month=4, day=15, tzinfo=utc),
         )
         afat_link_september = FatLink.objects.create(
             fleet="September Fleet",
             hash="1233",
             creator=cls.user_with_basic_access,
             character=cls.character_1001,
-            created=dt.datetime(year=2020, month=9, day=1, tzinfo=utc),
+            created=datetime(year=2020, month=9, day=1, tzinfo=utc),
         )
 
         Fat.objects.create(
@@ -138,12 +157,17 @@ class TestStatistics(TestCase):
         )
 
     def test_should_only_show_my_chars_and_only_those_with_fat_links(self):
-        # when
+        """
+        Test that the overview page only shows the characters of the user that have FAT links.
+
+        :return:
+        :rtype:
+        """
+
         result = _calculate_year_stats(
             request=RequestStub(user=self.user_with_basic_access), year=2020
         )
 
-        # then
         self.assertDictEqual(
             d1=result,
             d2={
@@ -156,60 +180,80 @@ class TestStatistics(TestCase):
         )
 
     def test_should_show_statistics_dashboard(self):
-        # given
+        """
+        Test should show statistics dashboard
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_basic_access)
 
-        # when
         url = reverse(viewname="afat:statistics_overview")
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_show_statistics_dashboard_for_year(self):
-        # given
+        """
+        Test should show statistics dashboard for year
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_basic_access)
 
-        # when
         url = reverse(viewname="afat:statistics_overview", kwargs={"year": 2020})
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_show_statistics_dashboard_for_user_with_stats_corporation_other(
         self,
     ):
-        # given
+        """
+        Test should show statistics dashboard for user with stats_corporation_other
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_stats_corporation_other)
 
-        # when
         url = reverse(viewname="afat:statistics_overview")
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_show_statistics_dashboard_for_user_with_stats_corporation_own(self):
-        # given
+        """
+        Test should show statistics dashboard for user with stats_corporation_own
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_stats_corporation_own)
 
-        # when
         url = reverse(viewname="afat:statistics_overview")
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_not_show_statistics_dashboard_for_user_without_access(self):
-        # given
+        """
+        Test should not show statistics dashboard for user without access
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_without_access)
 
-        # when
         url = reverse(viewname="afat:statistics_overview")
         res = self.client.get(path=url)
 
-        # then
         self.assertNotEqual(first=res.status_code, second=HTTPStatus.OK)
         self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)
         self.assertEqual(
@@ -218,10 +262,15 @@ class TestStatistics(TestCase):
         )
 
     def test_should_show_own_character_stats(self):
-        # given
+        """
+        Test should show own character stats
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_basic_access)
 
-        # when
         url = reverse(
             viewname="afat:statistics_character",
             kwargs={
@@ -232,16 +281,20 @@ class TestStatistics(TestCase):
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_show_other_character_stats_for_user_with_stats_corporation_own(
         self,
     ):
-        # given
+        """
+        Test should show other character stats for user with stats_corporation_own
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_stats_corporation_own)
 
-        # when
         url = reverse(
             viewname="afat:statistics_character",
             kwargs={
@@ -252,16 +305,20 @@ class TestStatistics(TestCase):
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_show_other_character_stats_for_user_with_stats_corporation_other(
         self,
     ):
-        # given
+        """
+        Test should show other character stats for user with stats_corporation_other
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_stats_corporation_other)
 
-        # when
         url = reverse(
             viewname="afat:statistics_character",
             kwargs={
@@ -272,14 +329,18 @@ class TestStatistics(TestCase):
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_show_other_character_stats_for_user_with_manage_afat(self):
-        # given
+        """
+        Test should show other character stats for user with manage_afat
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_manage_afat)
 
-        # when
         url = reverse(
             viewname="afat:statistics_character",
             kwargs={
@@ -290,14 +351,18 @@ class TestStatistics(TestCase):
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_not_show_other_character_stats_for_user(self):
-        # given
+        """
+        Test should not show other character stats for user without access
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_basic_access)
 
-        # when
         url = reverse(
             viewname="afat:statistics_character",
             kwargs={
@@ -308,14 +373,18 @@ class TestStatistics(TestCase):
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)
 
     def test_should_show_own_corp_stats_for_user_with_stats_corporation_own(self):
-        # given
+        """
+        Test should show own corp stats for user with stats_corporation_own
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_stats_corporation_own)
 
-        # when
         url = reverse(
             viewname="afat:statistics_corporation",
             kwargs={
@@ -324,14 +393,18 @@ class TestStatistics(TestCase):
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_show_other_corp_stats_for_user_with_stats_corporation_other(self):
-        # given
+        """
+        Test should show other corp stats for user with stats_corporation_other
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_stats_corporation_other)
 
-        # when
         url = reverse(
             viewname="afat:statistics_corporation",
             kwargs={
@@ -340,14 +413,18 @@ class TestStatistics(TestCase):
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_show_other_corp_stats_for_user_with_manage_afat(self):
-        # given
+        """
+        Test should show other corp stats for user with manage_afat
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_manage_afat)
 
-        # when
         url = reverse(
             viewname="afat:statistics_corporation",
             kwargs={
@@ -356,14 +433,18 @@ class TestStatistics(TestCase):
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_not_show_own_corp_stats_for_user(self):
-        # given
+        """
+        Test should not show own corp stats for user without access
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_basic_access)
 
-        # when
         url = reverse(
             viewname="afat:statistics_corporation",
             kwargs={
@@ -372,14 +453,18 @@ class TestStatistics(TestCase):
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)
 
     def test_should_not_show_other_corp_stats_for_user(self):
-        # given
+        """
+        Test should not show other corp stats for user without access
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_basic_access)
 
-        # when
         url = reverse(
             viewname="afat:statistics_corporation",
             kwargs={
@@ -388,113 +473,140 @@ class TestStatistics(TestCase):
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)
 
     def test_should_show_all_corp_stats_for_user_with_stats_corporation_other(self):
-        # given
+        """
+        Test should show all corp stats for user with stats_corporation_other
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_stats_corporation_other)
 
-        # when
         url = reverse(
             viewname="afat:statistics_corporation",
             kwargs={"corpid": 2002, "year": 2020},
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_show_all_corp_stats_with_month_for_user_with_stats_corporation_other(
         self,
     ):
-        # given
+        """
+        Test should show all corp stats with month for user with stats_corporation_other
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_stats_corporation_other)
 
-        # when
         url = reverse(
             viewname="afat:statistics_corporation",
             kwargs={"corpid": 2002, "year": 2020, "month": 4},
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_not_show_all_corp_stats_for_user_with_stats_corporation_own(self):
-        # given
+        """
+        Test should not show all corp stats for user with stats_corporation_own
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_stats_corporation_own)
 
-        # when
         url = reverse(viewname="afat:statistics_corporation", kwargs={"corpid": 2002})
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)
         self.assertEqual(first=res.url, second="/fleet-activity-tracking/")
 
     def test_should_show_all_alliance_stats_with_for_user_with_stats_corporation_other(
         self,
     ):
-        # given
+        """
+        Test should show all alliance stats for user with stats_corporation_other
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_stats_corporation_other)
 
-        # when
         url = reverse(
             viewname="afat:statistics_alliance",
             kwargs={"allianceid": 3001},
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_show_all_alliance_stats_with_for_user_with_manage_afat(
         self,
     ):
-        # given
+        """
+        Test should show all alliance stats for user with manage_afat
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_manage_afat)
 
-        # when
         url = reverse(
             viewname="afat:statistics_alliance",
             kwargs={"allianceid": 3001},
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_show_all_alliance_stats_with_year_for_user_with_stats_corporation_other(
         self,
     ):
-        # given
+        """
+        Test should show all alliance stats with year for user with stats_corporation_other
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_stats_corporation_other)
 
-        # when
         url = reverse(
             viewname="afat:statistics_alliance",
             kwargs={"allianceid": 3001, "year": 2020},
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_show_all_alliance_stats_with_year_for_user_with_manage_afat(
         self,
     ):
-        # given
+        """
+        Test should show all alliance stats with year for user with manage_afat
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_manage_afat)
 
-        # when
         url = reverse(
             viewname="afat:statistics_alliance",
             kwargs={"allianceid": 3001, "year": 2020},
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_show_all_alliance_stats_with_month_for_user_with_stats_corporation_other(
@@ -516,33 +628,41 @@ class TestStatistics(TestCase):
     def test_should_show_all_alliance_stats_with_month_for_user_with_manage_afat(
         self,
     ):
-        # given
+        """
+        Test should show all alliance stats with month for user with manage_afat
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_manage_afat)
 
-        # when
         url = reverse(
             viewname="afat:statistics_alliance",
             kwargs={"allianceid": 3001, "year": 2020, "month": 4},
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
 
     def test_should_not_show_all_alliance_stats_for_user(
         self,
     ):
-        # given
+        """
+        Test should not show all alliance stats for user without access
+
+        :return:
+        :rtype:
+        """
+
         self.client.force_login(user=self.user_with_basic_access)
 
-        # when
         url = reverse(
             viewname="afat:statistics_alliance",
             kwargs={"allianceid": 3001},
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)
 
     def test_should_show_main_details_for_user_with_manage_perms(self):
@@ -553,13 +673,12 @@ class TestStatistics(TestCase):
         :rtype:
         """
 
-        # given
         self.client.force_login(user=self.user_with_manage_afat)
 
-        # when
         url = reverse(
             viewname="afat:statistics_ajax_get_monthly_fats_for_main_character",
             kwargs={
+                "corporation_id": self.user_with_basic_access.profile.main_character.corporation_id,
                 "character_id": self.user_with_basic_access.profile.main_character.character_id,
                 "year": 2020,
                 "month": 4,
@@ -567,12 +686,7 @@ class TestStatistics(TestCase):
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
-        self.assertEqual(
-            first=response_content_to_str(response=res),
-            second='[{"character_id": 1002, "character_name": "Clark Kent", "fat_count": 2}]',
-        )
 
     def test_should_show_main_details_for_user_with_corporation_other_perms(self):
         """
@@ -582,13 +696,12 @@ class TestStatistics(TestCase):
         :rtype:
         """
 
-        # given
         self.client.force_login(user=self.user_with_stats_corporation_other)
 
-        # when
         url = reverse(
             viewname="afat:statistics_ajax_get_monthly_fats_for_main_character",
             kwargs={
+                "corporation_id": self.user_with_basic_access.profile.main_character.corporation_id,
                 "character_id": self.user_with_basic_access.profile.main_character.character_id,
                 "year": 2020,
                 "month": 4,
@@ -596,12 +709,7 @@ class TestStatistics(TestCase):
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.OK)
-        self.assertEqual(
-            first=response_content_to_str(response=res),
-            second='[{"character_id": 1002, "character_name": "Clark Kent", "fat_count": 2}]',
-        )
 
     def test_should_not_show_main_details_for_user_without_perms(self):
         """
@@ -611,13 +719,12 @@ class TestStatistics(TestCase):
         :rtype:
         """
 
-        # given
         self.client.force_login(user=self.user_with_basic_access)
 
-        # when
         url = reverse(
             viewname="afat:statistics_ajax_get_monthly_fats_for_main_character",
             kwargs={
+                "corporation_id": self.user_with_basic_access.profile.main_character.corporation_id,
                 "character_id": self.user_with_basic_access.profile.main_character.character_id,
                 "year": 2020,
                 "month": 4,
@@ -625,7 +732,6 @@ class TestStatistics(TestCase):
         )
         res = self.client.get(path=url)
 
-        # then
         self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)
         self.assertEqual(
             first=response_content_to_str(response=res),
