@@ -3,7 +3,7 @@ Fat links related views
 """
 
 # Standard Library
-from datetime import datetime, timedelta
+from datetime import timedelta
 
 # Django
 from django.contrib import messages
@@ -14,6 +14,7 @@ from django.http import HttpResponse, HttpResponseRedirect, JsonResponse
 from django.shortcuts import redirect, render
 from django.urls import reverse
 from django.utils import timezone
+from django.utils.datetime_safe import datetime
 from django.utils.safestring import mark_safe
 from django.utils.text import format_lazy
 from django.utils.translation import gettext_lazy as _
@@ -21,13 +22,13 @@ from django.utils.translation import gettext_lazy as _
 # Alliance Auth
 from allianceauth.authentication.decorators import permissions_required
 from allianceauth.eveonline.models import EveCharacter
-from allianceauth.eveonline.providers import provider
 from allianceauth.services.hooks import get_extension_logger
 from esi.decorators import token_required
 from esi.models import Token
 
 # Alliance Auth (External Libs)
 from app_utils.logging import LoggerAddTag
+from eveuniverse.models import EveSolarSystem, EveType
 
 # Alliance Auth AFAT
 from afat import __title__
@@ -37,6 +38,7 @@ from afat.forms import (
     AFatManualFatForm,
     FatLinkEditForm,
 )
+from afat.handler import esi_handler
 from afat.helper.fatlinks import get_doctrines, get_esi_fleet_information_by_user
 from afat.helper.time import get_time_delta
 from afat.helper.views import convert_fatlinks_to_dict, convert_fats_to_dict
@@ -133,14 +135,9 @@ def add_fatlink(request: WSGIRequest) -> HttpResponse:
     :rtype:
     """
 
-    fleet_types_configured = False
-    fleet_types = FleetType.objects.all()
-
-    if fleet_types.exists():
-        fleet_types_configured = True
+    fleet_types = FleetType.objects.filter(is_enabled=True).order_by("name")
 
     context = {
-        "fleet_types_configured": fleet_types_configured,
         "default_expiry_time": Setting.get_setting(
             Setting.Field.DEFAULT_FATLINK_EXPIRY_TIME
         ),
@@ -281,15 +278,16 @@ def create_esi_fatlink_callback(  # pylint: disable=too-many-locals
     """
 
     # Check if there is a fleet
+    required_scopes = ["esi-fleets.read_fleet.v1"]
+    esi_token = Token.get_token(character_id=token.character_id, scopes=required_scopes)
+    operation = esi.client.Fleets.GetCharactersCharacterIdFleet(
+        character_id=token.character_id, token=esi_token
+    )
     try:
-        required_scopes = ["esi-fleets.read_fleet.v1"]
-        esi_token = Token.get_token(
-            character_id=token.character_id, scopes=required_scopes
-        )
 
-        fleet_from_esi = esi.client.Fleets.get_characters_character_id_fleet(
-            character_id=token.character_id, token=esi_token.valid_access_token()
-        ).result()
+        fleet_from_esi = esi_handler.result(
+            operation=operation, return_cached_for_304=True
+        )
     except Exception:  # pylint: disable=broad-exception-caught
         # Not in a fleet
         messages.warning(
@@ -323,7 +321,7 @@ def create_esi_fatlink_callback(  # pylint: disable=too-many-locals
         character_has_registered_fleets = True
 
         for registered_fleet in registered_fleets_for_creator:
-            if registered_fleet.esi_fleet_id == fleet_from_esi["fleet_id"]:
+            if registered_fleet.esi_fleet_id == fleet_from_esi.fleet_id:
                 # Character already has a fleet
                 fleet_already_registered = True
             else:
@@ -344,7 +342,7 @@ def create_esi_fatlink_callback(  # pylint: disable=too-many-locals
                         "{creator__character_name} has already been registered and "
                         "pilots joining this fleet are automatically tracked.</p>"
                     ),
-                    fleet_id=fleet_from_esi["fleet_id"],
+                    fleet_id=fleet_from_esi.fleet_id,
                     creator__character_name=creator_character.character_name,
                 ),
             ),
@@ -377,11 +375,13 @@ def create_esi_fatlink_callback(  # pylint: disable=too-many-locals
             registered_fleet_to_close["registered_fleet"].save()
 
     # Check if we deal with the fleet boss here
+    operation = esi.client.Fleets.GetFleetsFleetIdMembers(
+        fleet_id=fleet_from_esi.fleet_id, token=esi_token
+    )
     try:
-        esi_fleet_member = esi.client.Fleets.get_fleets_fleet_id_members(
-            fleet_id=fleet_from_esi["fleet_id"],
-            token=esi_token.valid_access_token(),
-        ).result()
+        esi_fleet_member = esi_handler.result(
+            operation=operation, return_cached_for_304=True
+        )
     except Exception:  # pylint: disable=broad-exception-caught
         messages.warning(
             request=request,
@@ -410,7 +410,7 @@ def create_esi_fatlink_callback(  # pylint: disable=too-many-locals
         hash=fatlink_hash,
         is_esilink=True,
         is_registered_on_esi=True,
-        esi_fleet_id=fleet_from_esi["fleet_id"],
+        esi_fleet_id=fleet_from_esi.fleet_id,
         fleet_type=request.session["fatlink_form__type"],
     )
 
@@ -433,7 +433,7 @@ def create_esi_fatlink_callback(  # pylint: disable=too-many-locals
     logger.info(
         msg=(
             f'ESI FAT link "{fatlink_hash}" with name '
-            f'"{request.session["fatlink_form__name"]}"{fleet_type} '
+            f'"{request.session["fatlink_form__name"]}" {fleet_type} '
             f"was created by {request.user}"
         )
     )
@@ -444,7 +444,9 @@ def create_esi_fatlink_callback(  # pylint: disable=too-many-locals
 
     # Process fleet members in the background
     process_fats.delay(
-        data_list=esi_fleet_member, data_source="esi", fatlink_hash=fatlink_hash
+        data_list=[fleet_member.dict() for fleet_member in esi_fleet_member],
+        data_source="esi",
+        fatlink_hash=fatlink_hash,
     )
 
     messages.success(
@@ -514,11 +516,12 @@ def create_esi_fatlink(
         "esi-location.read_online.v1",
     ]
 )
-def add_fat(
-    request: WSGIRequest, token, fatlink_hash: str = None
+def add_fat(  # pylint: disable=too-many-locals
+    request: WSGIRequest, token, fatlink_hash: str
 ) -> HttpResponseRedirect:
     """
     Click fat link helper
+
     :param request:
     :type request:
     :param token:
@@ -529,16 +532,6 @@ def add_fat(
     :rtype:
     """
 
-    if fatlink_hash is None:
-        messages.warning(
-            request=request,
-            message=mark_safe(
-                s=_("<h4>Warning!</h4><p>No FAT link hash provided.</p>")
-            ),
-        )
-
-        return redirect(to="afat:dashboard")
-
     try:
         fleet = FatLink.objects.get(hash=fatlink_hash, is_esilink=False)
     except FatLink.DoesNotExist:
@@ -598,33 +591,44 @@ def add_fat(
         return redirect(to="afat:dashboard")
 
     # Check if character is online
-    character_online = esi.client.Location.get_characters_character_id_online(
-        character_id=token.character_id, token=esi_token.valid_access_token()
-    ).result()
+    operation = esi.client.Location.GetCharactersCharacterIdOnline(
+        character_id=token.character_id, token=esi_token
+    )
+    character_online = esi_handler.result(
+        operation=operation, return_cached_for_304=True
+    )
 
-    if character_online["online"] is True:
+    if character_online.online is True:
         # Character location
-        location = esi.client.Location.get_characters_character_id_location(
-            character_id=token.character_id,
-            token=esi_token.valid_access_token(),
-        ).result()
+        operation = esi.client.Location.GetCharactersCharacterIdLocation(
+            character_id=token.character_id, token=esi_token
+        )
+        location = esi_handler.result(operation=operation, return_cached_for_304=True)
+
+        solar_system, solar_system_created = (  # pylint: disable=unused-variable
+            EveSolarSystem.objects.get_or_create_esi(id=location.solar_system_id)
+        )
 
         # Current ship
-        ship = esi.client.Location.get_characters_character_id_ship(
-            character_id=token.character_id,
-            token=esi_token.valid_access_token(),
-        ).result()
+        operation = esi.client.Location.GetCharactersCharacterIdShip(
+            character_id=token.character_id, token=esi_token
+        )
+        current_ship = esi_handler.result(
+            operation=operation, return_cached_for_304=True
+        )
 
-        # System information
-        system = esi.client.Universe.get_universe_systems_system_id(
-            system_id=location["solar_system_id"]
-        ).result()["name"]
-
-        ship_name = provider.get_itemtype(type_id=ship["ship_type_id"]).name
+        ship, ship_created = (  # pylint: disable=unused-variable
+            EveType.objects.get_or_create_esi(id=current_ship.ship_type_id)
+        )
 
         try:
             Fat(
-                fatlink=fleet, character=character, system=system, shiptype=ship_name
+                fatlink=fleet,
+                character=character,
+                system=solar_system.name,
+                shiptype=ship.name,
+                corporation_eve_id=character.corporation_id,
+                alliance_eve_id=character.alliance_id,
             ).save()
         except IntegrityError:
             messages.warning(
@@ -916,9 +920,7 @@ def ajax_get_fats_by_fatlink(request: WSGIRequest, fatlink_hash) -> JsonResponse
 
 @login_required()
 @permission_required(perm="afat.manage_afat")
-def delete_fatlink(
-    request: WSGIRequest, fatlink_hash: str = None
-) -> HttpResponseRedirect:
+def delete_fatlink(request: WSGIRequest, fatlink_hash: str) -> HttpResponseRedirect:
     """
     Delete fat link helper
 
@@ -930,23 +932,13 @@ def delete_fatlink(
     :rtype:
     """
 
-    if fatlink_hash is None:
-        messages.warning(
-            request=request,
-            message=mark_safe(
-                s=_("<h4>Warning!</h4><p>No FAT link hash provided.</p>")
-            ),
-        )
-
-        return redirect(to="afat:dashboard")
-
     try:
         link = FatLink.objects.get(hash=fatlink_hash)
     except FatLink.DoesNotExist:
         messages.error(
-            request=request,
-            message=mark_safe(
-                s=_(
+            request,
+            mark_safe(
+                _(
                     "<h4>Error!</h4>"
                     "<p>The FAT link hash provided is either invalid "
                     "or the FAT link has already been deleted.</p>"
@@ -954,23 +946,24 @@ def delete_fatlink(
             ),
         )
 
-        return redirect(to="afat:dashboard")
+        return redirect("afat:dashboard")
 
+    # Delete associated FATs and the FAT link
     Fat.objects.filter(fatlink_id=link.pk).delete()
-
     link.delete()
 
+    # Log the deletion
     write_log(
-        request=request,
+        request,
         log_event=Log.Event.DELETE_FATLINK,
         log_text="FAT link deleted.",
-        fatlink_hash=link.hash,
+        fatlink_hash=fatlink_hash,
     )
 
     messages.success(
-        request=request,
-        message=mark_safe(
-            s=format_lazy(
+        request,
+        mark_safe(
+            format_lazy(
                 _(
                     "<h4>Success!</h4>"
                     '<p>The FAT link "{fatlink_hash}" and all associated FATs have '
@@ -982,13 +975,10 @@ def delete_fatlink(
     )
 
     logger.info(
-        msg=(
-            f'Fat link "{fatlink_hash}" and all associated '
-            f"FATs have been deleted by {request.user}"
-        )
+        f'Fat link "{fatlink_hash}" and all associated FATs have been deleted by {request.user}'
     )
 
-    return redirect(to="afat:fatlinks_overview")
+    return redirect("afat:fatlinks_overview")
 
 
 @login_required()
@@ -1011,6 +1001,7 @@ def delete_fat(
 
     try:
         link = FatLink.objects.get(hash=fatlink_hash)
+        fat_details = Fat.objects.get(pk=fat_id, fatlink_id=link.pk)
     except FatLink.DoesNotExist:
         messages.error(
             request=request,
@@ -1023,9 +1014,6 @@ def delete_fat(
         )
 
         return redirect(to="afat:dashboard")
-
-    try:
-        fat_details = Fat.objects.get(pk=fat_id, fatlink_id=link.pk)
     except Fat.DoesNotExist:
         messages.error(
             request=request,
@@ -1086,21 +1074,19 @@ def close_esi_fatlink(request: WSGIRequest, fatlink_hash: str) -> HttpResponseRe
 
     try:
         fatlink = FatLink.objects.get(hash=fatlink_hash)
-    except FatLink.DoesNotExist:
-        logger.info(msg=f'ESI FAT link with hash "{fatlink_hash}" does not exist')
-    else:
+        fatlink.is_registered_on_esi = False
+        fatlink.save()
+
         logger.info(
             msg=(
                 f'Closing ESI FAT link with hash "{fatlink_hash}". '
                 "Reason: Closed by manual request"
             )
         )
+    except FatLink.DoesNotExist:
+        logger.info(msg=f'ESI FAT link with hash "{fatlink_hash}" does not exist')
 
-        fatlink.is_registered_on_esi = False
-        fatlink.save()
-
-    default_redirect = reverse(viewname="afat:dashboard")
-    next_view = request.GET.get(key="next", default=default_redirect)
+    next_view = request.GET.get("next", reverse("afat:dashboard"))
 
     return HttpResponseRedirect(redirect_to=next_view)
 
@@ -1134,16 +1120,17 @@ def reopen_fatlink(request: WSGIRequest, fatlink_hash: str) -> HttpResponseRedir
 
         return redirect(to="afat:dashboard")
 
-    if fatlink_duration.fleet.reopened is False:
+    if not fatlink_duration.fleet.reopened:
         created_at = fatlink_duration.fleet.created
         now = datetime.now()
 
+        default_reopen_duration = Setting.get_setting(
+            Setting.Field.DEFAULT_FATLINK_REOPEN_DURATION
+        )
         time_difference_in_minutes = get_time_delta(
             then=created_at, now=now, interval="minutes"
         )
-        new_duration = time_difference_in_minutes + Setting.get_setting(
-            Setting.Field.DEFAULT_FATLINK_REOPEN_DURATION
-        )
+        new_duration = time_difference_in_minutes + default_reopen_duration
 
         fatlink_duration.duration = new_duration
         fatlink_duration.save()
@@ -1155,18 +1142,14 @@ def reopen_fatlink(request: WSGIRequest, fatlink_hash: str) -> HttpResponseRedir
         write_log(
             request=request,
             log_event=Log.Event.REOPEN_FATLINK,
-            log_text=(
-                "FAT link re-opened for a duration of "
-                f"{Setting.get_setting(Setting.Field.DEFAULT_FATLINK_REOPEN_DURATION)} minutes"
-            ),
+            log_text=f"FAT link re-opened for a duration of {default_reopen_duration} minutes",
             fatlink_hash=fatlink_duration.fleet.hash,
         )
 
         logger.info(
             msg=(
-                f'FAT link with hash "{fatlink_hash}" '
-                f"re-opened by {request.user} for a "
-                f"duration of {Setting.get_setting(Setting.Field.DEFAULT_FATLINK_REOPEN_DURATION)} minutes"
+                f'FAT link with hash "{fatlink_hash}" re-opened by {request.user} '
+                f"for a duration of {default_reopen_duration} minutes"
             )
         )
 
